//=============================================================================
//  Brass instrument fingering plugin for MuseScore
//
//  Copyright (C)2012 Ian McLeod (underquark)
//  Trombone positions by DerPosaunist
//  Copyright (c)2015 Joachim Schmitz
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//=============================================================================

import QtQuick 2.2
import MuseScore 1.0

MuseScore {
   version:  "1.0"
   description: "This plugin creates positions for trombone"
   menuPath: "Plugins.Create Trombone Positions"

   property variant notes1: [
      "7", "6", "5", "4", "3", "2", "1", // Bb //
      "7", "6", "5", "4", "3", "2", "1", // F //
      "5", "4", "3", "2", "1", // Bb //
      "4", "3", "2", "1", // D //
      "3", "2", "1",  // F //
      "3+", "2+", // G //
      "3", "2", "1", // Bb //
      "2", "1", // C //
      "2", "1", // D //
      "3", "2", "1",  // F //
      "3", "2", // G //
      "3", "2", "1"  // Bb //
      ]
//=============================================================================
// Alternative Positions
//=============================================================================
property variant notes2: [
      "7", "6", "5", "4", "3", "2", "1", // Bb //
      "7", "6", "5", "4", "3", "2/7", "1/6", // F //
      "5", "4", "3/7", "2/6", "1/5", // Bb //
      "4/7", "3/6", "2/5", "1/4/7", // D //
      "3/6", "2/5/7", "1/4/6",  // F //
      "3+/5/7", "2+/4/6", // G //
      "3/5/7", "2/4/6", "1/3/5", // Bb //
      "2/4/6", "1/3/5", // C //
      "2/4/5", "1/3/4", // D //
      "3/2", "2", "1",  // F //
      "3", "2", // G //
      "3", "2", "1"  // Bb //
      ]

   // add trombone position to top note of chord
   function func(topNote, text) {
      //var topNote = chord.notes[chord.lenght - 1]
      text.text = notes1[topNote.pitch - 40]
      //if (chord.topNote.pitch < 59)
      //   text.pos.y += 2 //
      }

   // Apply the given function to all topnotes in voice 1
   // in selection or, if nothing is selected, in the entire score
   function applyToChordsInSelection(func) {
      var cursor = curScore.newCursor()
      cursor.rewind(1)
      var startStaff
      var endStaff
      var endTick
      var fullScore = false
      if (!cursor.segment) { // no selection
         fullScore = true
         startStaff = 0 // start with 1st staff
         endStaff = curScore.nstaves - 1 // and end with last
         }
      else {
         startStaff = cursor.staffIdx
         cursor.rewind(2)
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1
            }
         else
            endTick = cursor.tick
         endStaff = cursor.staffIdx
         }
      console.log(startStaff + " - " + endStaff + " - " + endTick)
      for (var staff = startStaff; staff <= endStaff; staff++) {
         for (var voice = 0; voice < 1; voice++) { // voice 1 only
            cursor.rewind(1) // sets voice to 0
            cursor.voice = voice //voice has to be set after goTo
            cursor.staffIdx = staff

            if (fullScore)
               cursor.rewind(0) // if no selection, beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
               if (cursor.element && cursor.element.type == Element.CHORD) {
                  var text = newElement(Element.STAFF_TEXT)
                  var topNote = cursor.element.notes[cursor.element.notes.length - 1]
                  func(topNote, text)
                  cursor.add(text)
                  }
               cursor.next()
               }
            }
         }
      }

   onRun: {
      console.log("Hello, Trombone")
      if (typeof curScore !== 'undefined')
         applyToChordsInSelection(func)            
      Qt.quit()
      }
}
